using System.Threading.Tasks;
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Linq;

namespace WingStorageModule.Helper
{
    public class DownloadFileHelper
    {
        private static string _cacheFolderBase = "";

        /// <summary>
        /// 构造，初始化监听
        /// </summary>
        public static void InitDownloadFileHelper(HttpListener _listener,string folderBase,int httpPort)
        {
            //添加已经挂在的静态服务的监听
            _cacheFolderBase = folderBase;
            _listener.Prefixes.Add($"http://*:{httpPort}/");
            _listener.Start();
            Task.Run(async () =>
            {
                while (true)
                {
                    try
                    {
                        var context = await _listener.GetContextAsync().ConfigureAwait(false);
                        HandleHttpContext(context);
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.ToString());
                    }
                }
            });
        }

        /// <summary>
        /// 下载
        /// </summary>
        /// <param name="context">监听到的上下问对象</param>
        /// <returns>输出文件流</returns>
        private static void HandleHttpContext(HttpListenerContext context)
        {
            context.Response.Headers.Add("Access-Control-Allow-Origin", "*");
            if (context.Request.HttpMethod == "GET")
            {
                try
                {
                    //读地址
                    var uri = context.Request.Url;
                    if (uri != null)
                    {
                        var absPath = uri.AbsolutePath;
                        var downloadFileName = "";
                        var _cacheFolder = _cacheFolderBase;
                        var pathArray = absPath.Split("/");
                        if(pathArray != null && pathArray.Length > 3)
                        {
                            downloadFileName = pathArray[3];
                            _cacheFolder +=  pathArray[2] + "\\" + downloadFileName;
                        }
                        if(File.Exists(_cacheFolder))
                        {
                            //读文件
                            using (FileStream fs = new FileStream(_cacheFolder, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite))
                            {
                                byte[] bytes = new byte[fs.Length];
                                fs.Read(bytes, 0, bytes.Length);
                                //流输出
                                context.Response.ContentType = "application/octet-stream";
                                context.Response.StatusCode = (int)HttpStatusCode.OK;
                                context.Response.AddHeader("Content-Disposition", $"attachment;fileName={downloadFileName}");
                                try
                                {
                                    context.Response.ContentLength64 = bytes.Length;
                                    if (context.Response.OutputStream.CanWrite)
                                    {
                                        context.Response.OutputStream.Write(bytes, 0, bytes.Length);
                                    } 
                                    //输出结束并关闭流                               
                                    context.Response.OutputStream.Flush();
                                    context.Response.OutputStream.Close();
                                }
                                catch (Exception ex)
                                {
                                    if (ex is HttpListenerException httpListenerException)
                                    {
                                        if (httpListenerException.ErrorCode == 64)
                                        {
                                            //Logger.WriteLineWarn("[WriteReponseBytes]User aborted request.url:" + _context.Request.Url);
                                        }
                                        else
                                        {
                                            //Logger.WriteLineWarn("[WriteReponseBytes]HttpListenerException" + ex);
                                        }
                                    }
                                    else
                                    {
                                        //Logger.WriteLineError("[WriteReponseBytes]" + ex);
                                    }
                                }
                            }
                        }
                    }
                }
                catch(Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
        }
    }
}